﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;

namespace MSBuild.Mercurial.Command
{
    /// <summary>
    /// Implements the <see cref="IParameterBuilder"/> interface for
    /// the boolean properties.
    /// </summary>
    internal class ChoiceParameterBuilder : IParameterBuilder
    {
        /// <summary>
        /// Checks whether the managed property is a <see cref="bool"/>.
        /// </summary>
        /// <param name="propertyType">
        /// The checked property.
        /// </param>
        /// <returns>
        /// <c>true</c> if <paramref name="propertyType"/> is
        /// <see cref="bool"/>, <c>false</c> otherwise.
        /// </returns>
        public bool CanBuild(Type propertyType)
        {
            return typeof(bool).IsAssignableFrom(propertyType);
        }

        /// <summary>
        /// Builds the parameter associated with a boolean value.
        /// </summary>
        /// <param name="parameterPrefix">
        /// The prefix used for the parameter. If empty, the builded
        /// parameter is the default parameter.
        /// </param>
        /// <param name="propertyType">
        /// The type of the property.
        /// </param>
        /// <param name="propertyValue">
        /// The current value of the property. Can be <c>null</c>.
        /// </param>
        /// <returns>
        /// The <paramref name="parameterPrefix"/> if
        /// <paramref name="propertyValue"/> is <c>true</c>,
        /// <c>null</c> otherwise.
        /// </returns>
        /// <exception cref="NotSupportedException">
        /// The <paramref name="parameterPrefix"/> is <c>null</c> or empty.
        /// </exception>
        /// <exception cref="NotSupportedException">
        /// The <paramref name="propertyType"/> is not a boolean.
        /// </exception>
        public string Build(string parameterPrefix, Type propertyType, object propertyValue)
        {
            if (string.IsNullOrEmpty(parameterPrefix))
            {
                throw new NotSupportedException(Error.ChoiceBuilderDefaultParameter);
            }

            if (!this.CanBuild(propertyType))
            {
                throw new NotSupportedException(Error.ParameterBuilderCantBuild);
            }

            if (object.Equals(propertyValue, true))
            {
                return parameterPrefix;
            }
            else
            {
                return null;
            }
        }
    }
}
