﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Globalization;

namespace MSBuild.Mercurial.Command
{
    /// <summary>
    /// Implements the <see cref="IParameterBuilder"/> interface for
    /// the date properties.
    /// </summary>
    internal class DateParameterBuilder : IParameterBuilder
    {
        /// <summary>
        /// The format used to convert a date to its string representation
        /// for Mercurial.
        /// </summary>
        public const string HgDateFormat = "yyyy-MM-dd HH:mm:ss";

        /// <summary>
        /// Checks whether the managed property is a nullable
        /// <see cref="DateTime"/>.
        /// </summary>
        /// <param name="propertyType">
        /// The checked property.
        /// </param>
        /// <returns>
        /// <c>true</c> if <paramref name="propertyType"/> is
        /// a nullable <see cref="DateTime"/>, <c>false</c> otherwise.
        /// </returns>
        public bool CanBuild(Type propertyType)
        {
            return typeof(DateTime?).IsAssignableFrom(propertyType)
                && !typeof(DateTime).IsAssignableFrom(propertyType);
        }

        /// <summary>
        /// Builds the parameter associated with a date value.
        /// </summary>
        /// <param name="parameterPrefix">
        /// The prefix used for the parameter. If empty, the builded
        /// parameter is the default parameter.
        /// </param>
        /// <param name="propertyType">
        /// The type of the property.
        /// </param>
        /// <param name="propertyValue">
        /// The current value of the property. Can be <c>null</c>.
        /// </param>
        /// <returns>
        /// The string used to represent the current property in the
        /// Mercurial command.
        /// </returns>
        /// <exception cref="NotSupportedException">
        /// The <paramref name="propertyType"/> is not a nullable date.
        /// </exception>
        public string Build(string parameterPrefix, Type propertyType, object propertyValue)
        {
            if (!this.CanBuild(propertyType))
            {
                throw new NotSupportedException(Error.ParameterBuilderCantBuild);
            }

            DateTime? value = propertyValue as DateTime?;
            if (value == null)
            {
                return null;
            }
            else
            {
                string formated = value.Value.ToString(HgDateFormat, CultureInfo.InvariantCulture);
                formated = string.Format(CultureInfo.InvariantCulture, "\"{0}\"", formated);

                if (string.IsNullOrEmpty(parameterPrefix))
                {
                    return formated;
                }
                else
                {
                    return string.Format(CultureInfo.InvariantCulture, "{0} {1}", parameterPrefix, formated);
                }
            }
        }
    }
}
