﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Globalization;

namespace MSBuild.Mercurial.Command
{
    /// <summary>
    /// Implements the <see cref="IParameterBuilder"/> interface for
    /// the string properties.
    /// </summary>
    internal class StringParameterBuilder : IParameterBuilder
    {
        /// <summary>
        /// Escapes a string as a Mercurial parameter.
        /// </summary>
        /// <param name="value">
        /// The initial value.
        /// </param>
        /// <returns>
        /// The escaped string.
        /// </returns>
        public static string Escape(string value)
        {
            if (string.IsNullOrEmpty(value))
            {
                return value;
            }

            string result = value.Replace("\"", "\\\"");
            result = string.Format(CultureInfo.InvariantCulture, "\"{0}\"", result);

            return result;
        }

        /// <summary>
        /// Checks whether the managed property is a <see cref="string"/>.
        /// </summary>
        /// <param name="propertyType">
        /// The checked property.
        /// </param>
        /// <returns>
        /// <c>true</c> if <paramref name="propertyType"/> is
        /// <see cref="string"/>, <c>false</c> otherwise.
        /// </returns>
        public bool CanBuild(Type propertyType)
        {
            return typeof(string).IsAssignableFrom(propertyType);
        }

        /// <summary>
        /// Builds the parameter associated with a string value.
        /// </summary>
        /// <param name="parameterPrefix">
        /// The prefix used for the parameter. If empty, the builded
        /// parameter is the default parameter.
        /// </param>
        /// <param name="propertyType">
        /// The type of the property.
        /// </param>
        /// <param name="propertyValue">
        /// The current value of the property. Can be <c>null</c>.
        /// </param>
        /// <returns>
        /// The string used to represent the current property in the
        /// Mercurial command.
        /// </returns>
        /// <exception cref="NotSupportedException">
        /// The <paramref name="propertyType"/> is not a string.
        /// </exception>
        public string Build(string parameterPrefix, Type propertyType, object propertyValue)
        {
            if (!this.CanBuild(propertyType))
            {
                throw new NotSupportedException(Error.ParameterBuilderCantBuild);
            }

            string value = propertyValue as string;
            if (!string.IsNullOrEmpty(value))
            {
                value = Escape(value);

                if (string.IsNullOrEmpty(parameterPrefix))
                {
                    return value;
                }
                else
                {
                    return string.Format(CultureInfo.InvariantCulture, "{0} {1}", parameterPrefix, value);
                }
            }
            else
            {
                return null;
            }
        }
    }
}
