﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;
using NUnit.Framework;

namespace MSBuild.Mercurial.Command
{
    /// <summary>
    /// Tests the <see cref="ParameterBuilderFactory"/> class.
    /// </summary>
    [TestFixture]
    public static class ParameterBuilderFactoryTest
    {
        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.Instance"/> property.
        /// </summary>
        [Test]
        public static void Instance()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;

            // Check that the instance is created on first call.
            Assert.IsNotNull(instance);

            // Check that the instance is not re-created
            ParameterBuilderFactory second = ParameterBuilderFactory.Instance;
            Assert.AreSame(instance, second);
        }

        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.Builders"/> property.
        /// </summary>
        [Test]
        public static void Builders()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;

            Assert.IsNotNull(instance.Builders);
            Assert.AreEqual(4, instance.Builders.Length);

            foreach (IParameterBuilder builder in instance.Builders)
            {
                Assert.IsNotNull(builder);
            }
        }

        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.CanBuild"/> method.
        /// </summary>
        [Test]
        public static void CanBuild()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;
            Assert.IsTrue(instance.CanBuild(typeof(string)));
            Assert.IsTrue(instance.CanBuild(typeof(bool)));
            Assert.IsTrue(instance.CanBuild(typeof(DateTime?)));
            Assert.IsTrue(instance.CanBuild(typeof(ITaskItem[])));
            Assert.IsFalse(instance.CanBuild(typeof(int)));
            Assert.IsFalse(instance.CanBuild(null));
        }

        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.Build"/> method with a string value.
        /// </summary>
        [Test]
        public static void BuildString()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;
            string actual;

            actual = instance.Build("-f", typeof(string), "value");
            Assert.AreEqual("-f \"value\"", actual);
        }

        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.Build"/> method with a boolean value.
        /// </summary>
        [Test]
        public static void BuildChoice()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;
            string actual;

            actual = instance.Build("-f", typeof(bool), true);
            Assert.AreEqual("-f", actual);
        }

        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.Build"/> method with a datetime value.
        /// </summary>
        [Test]
        public static void BuildDate()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;
            string actual;

            DateTime? reference = new DateTime(2000, 1, 2, 13, 4, 5);
            string expected = "\"2000-01-02 13:04:05\"";
            actual = instance.Build("-f", typeof(DateTime?), reference);
            Assert.AreEqual("-f " + expected, actual);
        }

        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.Build"/> method with an array of items.
        /// </summary>
        [Test]
        public static void BuildItems()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;
            ITaskItem[] reference = new ITaskItem[]
            {
                new TaskItem("File1.cs"),
                new TaskItem("Dir\\File2.cs"),
            };
            string actual;

            actual = instance.Build(null, typeof(ITaskItem[]), reference);
            Assert.AreEqual("\"File1.cs\" \"Dir\\File2.cs\"", actual);
        }

        /// <summary>
        /// Tests the <see cref="ParameterBuilderFactory.Build"/> method
        /// with a wrong property type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(NotSupportedException))]
        public static void BuildWrongType()
        {
            ParameterBuilderFactory instance = ParameterBuilderFactory.Instance;
            instance.Build("-f", typeof(int), 1);
        }
    }
}
