﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests the <see cref="CommandBuilder"/> class.
    /// </summary>
    [TestFixture]
    public static class CommandBuilderTest
    {
        /// <summary>
        /// Tests the <see cref="CommandBuilder.Build"/> method.
        /// </summary>
        [Test]
        public static void Build()
        {
            MockTask task = new MockTask();
            task.BoolParameter = true;
            task.StringParameter = "my \\ string";
            task.DateTimeParameter = new DateTime(2000, 1, 2, 13, 4, 5);
            task.DefaultParameter = "default parameter";
            string actual = CommandBuilder.Build(task);

            Assert.IsNotNull(actual);

            // Valid the commmand
            StringAssert.Contains("mock", actual);

            // Valid the boolean parameter
            StringAssert.Contains("bool", actual);

            // Valid the string parameter
            StringAssert.Contains("string \"my \\ string\"", actual);

            // Valid the date-time parameter
            // format: yyyy-MM-dd HH:mm:ss
            StringAssert.Contains("date \"2000-01-02 13:04:05\"", actual);

            // Valid the default parameter
            StringAssert.EndsWith("\"default parameter\"", actual);

            // Valid that there is no extra parameter or double data
            string other = actual
                .Replace("mock", "*")
                .Replace("bool", "-")
                .Replace("string \"my \\ string\"", "-")
                .Replace("date \"2000-01-02 13:04:05\"", "-")
                .Replace("\"default parameter\"", "+");

            Assert.AreEqual("* - - - +", other);
        }

        /// <summary>
        /// Tests the <see cref="CommandBuilder.Build"/> method
        /// with an empty task.
        /// </summary>
        [Test]
        public static void BuildEmptyArgument()
        {
            MockTask task = new MockTask();
            string emptyCommand = CommandBuilder.Build(task);

            Assert.IsNotNull(emptyCommand);
            Assert.AreEqual("mock", emptyCommand);
        }

        /// <summary>
        /// Tests the <see cref="CommandBuilder.Build"/> method
        /// with a <c>null</c> parameter.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public static void BuildNullParameter()
        {
            CommandBuilder.Build(null);
        }
    }
}
