﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.IO;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests the <see cref="HgAdd"/> class.
    /// </summary>
    [TestFixture]
    public static class HgAddTest
    {
        /// <summary>
        /// The name of the first file used for tests.
        /// </summary>
        public const string TestFile1 = "test-add.txt";

        /// <summary>
        /// The name of the second file used for tests.
        /// </summary>
        public const string TestFile2 = "test-add.csv";

        /// <summary>
        /// Gets the full path of the first file used for tests.
        /// </summary>
        public static string TestFilePath1
        {
            get
            {
                return Path.Combine(Repository.FullPath, TestFile1);
            }
        }

        /// <summary>
        /// Gets the full path of the second file used for tests.
        /// </summary>
        public static string TestFilePath2
        {
            get
            {
                return Path.Combine(Repository.FullPath, TestFile2);
            }
        }

        /// <summary>
        /// Initializes the repository used for tests and the new file.
        /// </summary>
        [SetUp]
        public static void SetUp()
        {
            Repository.Create();

            // Create the test files
            File.WriteAllText(TestFilePath1, "test");
            File.WriteAllText(TestFilePath2, "test");
        }

        /// <summary>
        /// Deletes the repository used for tests.
        /// </summary>
        [TearDown]
        public static void TearDown()
        {
            Repository.Delete();
        }

        /// <summary>
        /// Tests the <c>HgAdd.Execute</c> method without parameter.
        /// </summary>
        [Test]
        public static void ExecuteWithoutParameter()
        {
            HgAdd instance = new HgAdd();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;

            string[] output;
            output = Repository.Execute("status");
            OutputAssert.Contains(output, "?", TestFile1);
            OutputAssert.Contains(output, "?", TestFile2);

            Assert.IsTrue(instance.Execute());

            output = Repository.Execute("status");
            OutputAssert.Contains(output, "A", TestFile1);
            OutputAssert.Contains(output, "A", TestFile2);
        }

        /// <summary>
        /// Tests the <c>HgAdd.Execute</c> method with the
        /// <see cref="HgAdd.DryRun"/> parameter.
        /// </summary>
        [Test]
        public static void ExecuteWithDryRun()
        {
            HgAdd instance = new HgAdd();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.DryRun = true;

            string[] output;
            output = Repository.Execute("status");
            OutputAssert.Contains(output, "?", TestFile1);
            OutputAssert.Contains(output, "?", TestFile2);

            Assert.IsTrue(instance.Execute());

            output = Repository.Execute("status");
            OutputAssert.Contains(output, "?", TestFile1);
            OutputAssert.Contains(output, "?", TestFile2);
        }

        /// <summary>
        /// Tests the <c>HgAdd.Execute</c> method with the
        /// <see cref="HgAdd.Include"/> parameter.
        /// </summary>
        [Test]
        public static void ExecuteWithInclude()
        {
            HgAdd instance = new HgAdd();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Include = "*.txt";

            string[] output;
            output = Repository.Execute("status");
            OutputAssert.Contains(output, "?", TestFile1);
            OutputAssert.Contains(output, "?", TestFile2);

            Assert.IsTrue(instance.Execute());

            output = Repository.Execute("status");
            OutputAssert.Contains(output, "A", TestFile1);
            OutputAssert.Contains(output, "?", TestFile2);
        }

        /// <summary>
        /// Tests the <c>HgAdd.Execute</c> method with the
        /// <see cref="HgAdd.Exclude"/> parameter.
        /// </summary>
        [Test]
        public static void ExecuteWithExclude()
        {
            HgAdd instance = new HgAdd();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Exclude = "*.txt";

            string[] output;
            output = Repository.Execute("status");
            OutputAssert.Contains(output, "?", TestFile1);
            OutputAssert.Contains(output, "?", TestFile2);

            Assert.IsTrue(instance.Execute());

            output = Repository.Execute("status");
            OutputAssert.Contains(output, "?", TestFile1);
            OutputAssert.Contains(output, "A", TestFile2);
        }

        /// <summary>
        /// Tests the <c>HgAdd.Execute</c> method with the
        /// <see cref="HgAdd.Files"/> parameter.
        /// </summary>
        [Test]
        public static void ExecuteWithFiles()
        {
            HgAdd instance = new HgAdd();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Files = new ITaskItem[]
            {
                new TaskItem(TestFile1),
                new TaskItem(TestFile2),
            };

            string[] output;
            output = Repository.Execute("status");
            OutputAssert.Contains(output, "?", TestFile1);
            OutputAssert.Contains(output, "?", TestFile2);

            Assert.IsTrue(instance.Execute());

            output = Repository.Execute("status");
            OutputAssert.Contains(output, "A", TestFile1);
            OutputAssert.Contains(output, "A", TestFile2);
        }
    }
}
