﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests the <see cref="HgClone"/> class.
    /// </summary>
    [TestFixture]
    public static class HgCloneTest
    {
        /// <summary>
        /// Initializes the repository to be cloned.
        /// </summary>
        [SetUp]
        public static void SetUp()
        {
            Repository.Create();
        }

        /// <summary>
        /// Deletes the created repositories.
        /// </summary>
        [TearDown]
        public static void TearDown()
        {
            Repository.Delete();
        }

        /// <summary>
        /// Tests the <see cref="HgClone.Destination"/> property.
        /// </summary>
        [Test]
        public static void Destination()
        {
            HgClone instance = new HgClone();
            Assert.IsNull(instance.Destination);

            instance.LocalPath = "my path";
            Assert.AreEqual("my path", instance.Destination);

            instance.Destination = "my path 2";
            Assert.AreEqual("my path 2", instance.LocalPath);
        }

        /// <summary>
        /// Tests the <c>HgClone.Execute</c> method with a local repository.
        /// </summary>
        [Test]
        public static void ExecuteLocal()
        {
            HgClone instance = new HgClone();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath2;
            instance.Source = Repository.FullPath;

            bool result = instance.Execute();

            Assert.IsTrue(result);

            string[] output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "0:");
        }

        /// <summary>
        /// Tests the <c>HgClone.Execute</c> method with a remote repository.
        /// </summary>
        [Test]
        public static void ExecuteRemote()
        {
            MockRemoteRepository.ResetRepository();
            HgClone instance = new HgClone();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath2;
            instance.Source = MockRemoteRepository.RepositoryUrl;
            instance.User = "test-read";
            instance.Password = "test";

            bool result = instance.Execute();

            Assert.IsTrue(result);

            string[] output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "0:");
        }

        /// <summary>
        /// Tests the <c>HgClone.Execute</c> method with a remote repository
        /// without credential.
        /// </summary>
        [Test]
        public static void ExecuteRemoteWithoutCredential()
        {
            MockRemoteRepository.ResetRepository();
            HgClone instance = new HgClone();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath2;
            instance.Source = MockRemoteRepository.RepositoryUrl;

            bool result = instance.Execute();

            Assert.IsFalse(result);
        }
    }
}
