﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.IO;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests the <see cref="HgCommit"/> class.
    /// </summary>
    [TestFixture]
    public static class HgCommitTest
    {
        /// <summary>
        /// Initializes a repository used to test the HgCommit task.
        /// </summary>
        /// <remarks>
        /// This method updates the existing file of the repository and adds a new file
        /// named <c>file2.txt</c> without adding it to the repository.
        /// </remarks>
        [SetUp]
        public static void SetUp()
        {
            Repository.Create();

            string[] output;

            output = Repository.Execute("summary");
            OutputAssert.Contains(output, "parent:", "0:");

            File.AppendAllText(Repository.FilePath, "my text");
            File.WriteAllText(Path.Combine(Repository.FullPath, "file2.txt"), "my text");

            output = Repository.Execute("summary");
            OutputAssert.Contains(output, "parent:", "0:");
            OutputAssert.Contains(output, "commit:", "1 modified, 1 unknown");
        }

        /// <summary>
        /// Deletes the repository used for the tests.
        /// </summary>
        [TearDown]
        public static void TearDown()
        {
            Repository.Delete();
        }

        /// <summary>
        /// Tests the <c>HgCommit.Execute</c> method with a message.
        /// </summary>
        [Test]
        public static void ExecuteMessage()
        {
            HgCommit instance = new HgCommit();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Message = "my new commit";

            Assert.IsTrue(instance.Execute());

            string[] output = Repository.Execute("summary");
            OutputAssert.Contains(output, "parent:", "1:");
            OutputAssert.Contains(output, "commit:", "1 unknown");
            OutputAssert.Contains(output, "my new commit");
        }

        /// <summary>
        /// Tests the <c>HgCommit.Execute</c> method with a message in an external file.
        /// </summary>
        [Test]
        public static void ExecuteLogFile()
        {
            HgCommit instance = new HgCommit();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.LogFile = "hgcommit-log.txt";

            File.WriteAllText(Path.Combine(Repository.FullPath, "hgcommit-log.txt"), "my new commit");

            Assert.IsTrue(instance.Execute());

            string[] output = Repository.Execute("summary");
            OutputAssert.Contains(output, "parent:", "1:");
            OutputAssert.Contains(output, "commit:", "2 unknown");
            OutputAssert.Contains(output, "my new commit");
        }

        /// <summary>
        /// Tests the <c>HgCommit.Execute</c> method with the AddRemove parameter.
        /// </summary>
        [Test]
        public static void ExecuteAddRemove()
        {
            HgCommit instance = new HgCommit();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Message = "my new commit";
            instance.AddRemove = true;

            Assert.IsTrue(instance.Execute());

            string[] output = Repository.Execute("summary");
            OutputAssert.Contains(output, "parent:", "1:");
            OutputAssert.DoesNotContain(output, "commit:", "unknown");
            OutputAssert.Contains(output, "my new commit");
        }

        /// <summary>
        /// Tests the <c>HgCommit.Execute</c> method with no message.
        /// </summary>
        [Test]
        public static void ExecuteNoMessage()
        {
            HgCommit instance = new HgCommit();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;

            Assert.IsFalse(instance.Execute());

            string[] output = Repository.Execute("summary");
            OutputAssert.Contains(output, "parent:", "0:");
        }
    }
}
