﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests of the <see cref="HgProcess"/> class.
    /// </summary>
    [TestFixture]
    public static class HgProcessTest
    {
        /// <summary>
        /// Tests of the <see cref="HgProcess"/> constructor.
        /// </summary>
        [Test]
        public static void Constructor()
        {
            string expectedLocation = "my library location";
            string expectedCommand = "my command";
            string expectedDirectory = "my working directory";

            HgProcess instance = new HgProcess(expectedLocation, expectedCommand, expectedDirectory);

            Assert.AreEqual(expectedCommand, instance.Command);
            Assert.AreEqual(expectedLocation, instance.LibraryLocation);
            Assert.AreEqual(expectedDirectory, instance.WorkingDirectory);
        }

        /// <summary>
        /// Tests of the <see cref="HgProcess.LibraryLocation"/> property
        /// with a <c>null</c> value.
        /// </summary>
        [Test]
        public static void LibraryLocationNull()
        {
            string expectedCommand = "my command";
            string expectedDirectory = "my working directory";

            HgProcess instance = new HgProcess(null, expectedCommand, expectedDirectory);

            Assert.AreEqual(null, instance.LibraryLocation);
        }

        /// <summary>
        /// Tests of the <see cref="HgProcess.Command"/> property
        /// with a <c>null</c> value.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public static void CommandNull()
        {
            string expectedLocation = "my library location";
            string expectedDirectory = "my working directory";

            HgProcess instance = new HgProcess(expectedLocation, null, expectedDirectory);

            // This test is only here to avoid FxCop warning.
            // In fact, the previous call should raise an exception.
            Assert.IsNotNull(instance);
        }

        /// <summary>
        /// Tests of the <see cref="HgProcess.WorkingDirectory"/> property
        /// with a <c>null</c> value.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public static void WorkingDirectoryNull()
        {
            string expectedLocation = "my library location";
            string expectedCommand = "my command";

            HgProcess instance = new HgProcess(expectedLocation, expectedCommand, null);

            // This test is only here to avoid FxCop warning.
            // In fact, the previous call should raise an exception.
            Assert.IsNotNull(instance);
        }

        /// <summary>
        /// Tests of the <see cref="HgProcess.StartAndWait"/> method.
        /// </summary>
        [Test]
        public static void StartAndWait()
        {
            using (HgProcess instance = new HgProcess(null, "summary", Environment.CurrentDirectory))
            {
                Assert.AreEqual(HgProcessStatus.Created, instance.Status);

                instance.StartAndWait(5000);

                Assert.AreEqual(HgProcessStatus.Ended, instance.Status);
                Assert.AreEqual(0, instance.Errors.Count);
                Assert.Greater(instance.Outputs.Count, 1);
                StringAssert.StartsWith("parent: ", instance.Outputs.ElementAt(0));
            }
        }

        /// <summary>
        /// Tests of the <see cref="HgProcess.Start"/> method.
        /// </summary>
        [Test]
        public static void Start()
        {
            using (HgProcess instance = new HgProcess(null, "summary", Environment.CurrentDirectory))
            {
                Assert.AreEqual(HgProcessStatus.Created, instance.Status);
                instance.Start();
                Assert.AreEqual(HgProcessStatus.Started, instance.Status);
            }
        }

        /// <summary>
        /// Tests of the <see cref="HgProcess.Wait"/> method.
        /// </summary>
        [Test]
        public static void Wait()
        {
            using (HgProcess instance = new HgProcess(null, "summary", Environment.CurrentDirectory))
            {
                Assert.AreEqual(HgProcessStatus.Created, instance.Status);
                instance.Start();
                Assert.AreEqual(HgProcessStatus.Started, instance.Status);
                instance.Wait(2000);
                Assert.AreEqual(HgProcessStatus.Ended, instance.Status);
                Assert.AreEqual(0, instance.Errors.Count);
            }
        }
    }
}
