﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests the <see cref="HgPull"/> class.
    /// </summary>
    [TestFixture]
    public static class HgPullTest
    {
        /// <summary>
        /// Initializes a repository used to test the HgPull task.
        /// </summary>
        [SetUp]
        public static void SetUp()
        {
            Repository.Create();
            Repository.Create2();
        }

        /// <summary>
        /// Deletes the repository used for the tests.
        /// </summary>
        [TearDown]
        public static void TearDown()
        {
            Repository.Delete();
        }

        /// <summary>
        /// Tests the <c>HgPull.Execute</c> method on local repository.
        /// </summary>
        /// <param name="withUpdate">
        /// A value indicating whether the <see cref="HgPull.Update"/>
        /// should be set to <c>true</c> for the test.
        /// </param>
        [TestCase(true, TestName = "with update")]
        [TestCase(false, TestName = "without update")]
        public static void ExecuteLocal(bool withUpdate)
        {
            HgPull instance = new HgPull();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath2;
            instance.Source = Repository.FullPath;
            instance.Update = withUpdate;

            string[] output;

            output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "-1:");

            Assert.IsTrue(instance.Execute());

            if (!withUpdate)
            {
                Repository.Execute2("update");
            }

            output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "0:");
        }

        /// <summary>
        /// Tests the <c>HgPull.Execute</c> method on remote repository.
        /// </summary>
        /// <param name="withUpdate">
        /// A value indicating whether the <see cref="HgPull.Update"/>
        /// should be set to <c>true</c> for the test.
        /// </param>
        [TestCase(true, TestName = "with update")]
        [TestCase(false, TestName = "without update")]
        public static void ExecuteRemote(bool withUpdate)
        {
            MockRemoteRepository.ResetRepository();
            HgPull instance = new HgPull();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath2;
            instance.Source = MockRemoteRepository.RepositoryUrl;
            instance.User = "test-read";
            instance.Password = "test";
            instance.Update = withUpdate;

            string[] output;

            output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "-1:");

            Assert.IsTrue(instance.Execute());

            if (!withUpdate)
            {
                Repository.Execute2("update");
            }

            output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "0:");
        }

        /// <summary>
        /// Tests the <c>HgPull.Execute</c> method on remote repository
        /// without credential.
        /// </summary>
        [Test]
        public static void ExecuteRemoteWithoutCredential()
        {
            MockRemoteRepository.ResetRepository();
            HgPull instance = new HgPull();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath2;
            instance.Source = MockRemoteRepository.RepositoryUrl;

            string[] output;

            output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "-1:");

            Assert.IsFalse(instance.Execute());

            Repository.Execute2("update");
            output = Repository.Execute2("summary");
            OutputAssert.Contains(output, "parent:", "-1:");
        }
    }
}
