﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Globalization;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests the <see cref="HgTag"/> class.
    /// </summary>
    [TestFixture]
    public static class HgTagTest
    {
        /// <summary>
        /// Initializes a repository used to test the HgTag task.
        /// </summary>
        [SetUp]
        public static void SetUp()
        {
            Repository.Create();
        }

        /// <summary>
        /// Deletes the repository used for the tests.
        /// </summary>
        [TearDown]
        public static void TearDown()
        {
            Repository.Delete();
        }

        /// <summary>
        /// Tests the <c>HgTag.Execute</c> method.
        /// </summary>
        [Test]
        public static void Execute()
        {
            HgTag instance = new HgTag();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Tag = "v1.0";

            string[] before = Repository.Execute("log -r0");
            OutputAssert.DoesNotContain(before, "tag:", "v1.0");

            Assert.IsTrue(instance.Execute());

            string[] result = Repository.Execute("log -r0");
            OutputAssert.Contains(result, "tag:", "v1.0");
        }

        /// <summary>
        /// Tests the <c>HgTag.Execute</c> method
        /// with most of its arguments.
        /// </summary>
        [Test]
        public static void ExecuteWithArguments()
        {
            HgTag instance = new HgTag();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Message = "my message";
            instance.Date = DateTime.Now;
            instance.User = "my user";
            instance.Revision = "0";
            instance.Tag = "v1.0";

            string[] before = Repository.Execute("log -r0");
            OutputAssert.DoesNotContain(before, "tag:", "v1.0");

            Assert.IsTrue(instance.Execute());

            string[] after = Repository.Execute("log -r0");
            OutputAssert.Contains(after, "tag:", "v1.0");

            string[] result = Repository.Execute("log -r1");
            OutputAssert.Contains(result, "user:", "my user");
            OutputAssert.Contains(result, "summary:", "my message");
            OutputAssert.Contains(result, "date:", instance.Date.Value.ToString("ddd MMM dd HH:mm:ss yyyy", CultureInfo.InvariantCulture));
        }

        /// <summary>
        /// Tests the <c>HgTag.Execute</c> method
        /// with the <c>local</c> argument.
        /// </summary>
        [Test]
        public static void ExecuteLocal()
        {
            HgTag instance = new HgTag();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Local = true;
            instance.Tag = "v1.0";

            string[] before = Repository.Execute("log -r0");
            OutputAssert.DoesNotContain(before, "tag:", "v1.0");

            Assert.IsTrue(instance.Execute());

            string[] after = Repository.Execute("log -r0");
            OutputAssert.Contains(after, "tag:", "v1.0");
            OutputAssert.Contains(after, "tag:", "tip");
        }

        /// <summary>
        /// Tests the <c>HgTag.Execute</c> method
        /// with the <c>force</c> argument.
        /// </summary>
        [Test]
        public static void ExecuteForce()
        {
            HgTag instance = new HgTag();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Tag = "v1.0";

            string[] output;
            output = Repository.Execute("log -r0");
            OutputAssert.DoesNotContain(output, "tag:", "v1.0");

            Assert.IsTrue(instance.Execute());

            output = Repository.Execute("log -r0");
            OutputAssert.Contains(output, "tag:", "v1.0");

            instance.Force = true;
            Assert.IsTrue(instance.Execute());

            output = Repository.Execute("log -r0");
            OutputAssert.DoesNotContain(output, "tag:", "v1.0");

            output = Repository.Execute("log -r1");
            OutputAssert.Contains(output, "tag:", "v1.0");
        }
    }
}
