﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.IO;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Tests the <see cref="HgUpdate"/> class.
    /// </summary>
    [TestFixture]
    public static class HgUpdateTest
    {
        /// <summary>
        /// The name of the first file used for tests.
        /// </summary>
        public const string TestFile1 = "test-update.txt";

        /// <summary>
        /// Gets the full path of the first file used for tests.
        /// </summary>
        public static string TestFilePath1
        {
            get
            {
                return Path.Combine(Repository.FullPath, TestFile1);
            }
        }

        /// <summary>
        /// Initializes the repository used for tests.
        /// </summary>
        [SetUp]
        public static void SetUp()
        {
            Repository.Create();

            File.WriteAllText(TestFilePath1, "my test");
            Repository.Execute("add " + TestFile1);
            Repository.Execute("commit -m \"test commit\"");
        }

        /// <summary>
        /// Deletes the repository used for tests.
        /// </summary>
        [TearDown]
        public static void TearDown()
        {
            Repository.Delete();
        }

        /// <summary>
        /// Tests the <c>HgUpdate.Execute</c> method without arguments.
        /// </summary>
        [Test]
        public static void Execute()
        {
            HgUpdate instance = new HgUpdate();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Revision = "0";

            Assert.IsTrue(instance.Execute());
            Assert.IsFalse(File.Exists(TestFilePath1));

            instance.Revision = null;

            Assert.IsTrue(instance.Execute());
            Assert.IsTrue(File.Exists(TestFilePath1));
        }

        /// <summary>
        /// Tests the <c>HgUpdate.Execute</c> method with the
        /// <see cref="HgUpdate.Check"/> parameter.
        /// </summary>
        [Test]
        public static void ExecuteCheck()
        {
            HgUpdate instance = new HgUpdate();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Check = true;
            instance.Revision = "0";

            Assert.IsTrue(instance.Execute());
            Assert.IsFalse(File.Exists(TestFilePath1));

            instance.Revision = null;
            File.AppendAllText(Repository.FilePath, "test");

            Assert.IsFalse(instance.Execute());
            Assert.IsFalse(File.Exists(TestFilePath1));
        }

        /// <summary>
        /// Tests the <c>HgUpdate.Execute</c> method with the
        /// <see cref="HgUpdate.Clean"/> parameter.
        /// </summary>
        [Test]
        public static void ExecuteClean()
        {
            HgUpdate instance = new HgUpdate();
            instance.BuildEngine = new MockBuildEngine();
            instance.LocalPath = Repository.FullPath;
            instance.Clean = true;
            instance.Revision = "0";

            Assert.IsTrue(instance.Execute());
            Assert.IsFalse(File.Exists(TestFilePath1));

            instance.Revision = null;
            File.AppendAllText(Repository.FilePath, "update-test");

            Assert.IsTrue(instance.Execute());
            StringAssert.DoesNotEndWith("update-test", File.ReadAllText(Repository.FilePath));
        }
    }
}
