﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Collections;
using System.Diagnostics;
using Microsoft.Build.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Simulates a BuildEngine instance.
    /// </summary>
    /// <remarks>
    /// <para>
    /// This class was inspired by the class used in the
    /// MSBuild Community Tasks especially for default values
    /// of properties: see
    /// (http://msbuildtasks.tigris.org/svn/msbuildtasks/trunk/Source/MSBuild.Community.Tasks.Tests/MockBuild.cs)
    /// </para>
    /// <para>
    /// Significantly changes are:
    /// <list type="bullet">
    /// <item>Implements the <see cref="IBuildEngine2"/> interface</item>
    /// <item>Use of a TraceSource for logging purposes</item>
    /// <item>No count of logged messages</item>
    /// <item>Documentation of the members :)</item>
    /// </list>
    /// </para>
    /// </remarks>
    public class MockBuildEngine : IBuildEngine2
    {
        /// <summary>
        /// Gets a value indicating whether the ContinueOnError flag was set
        /// to <c>true</c> for this particular task in the project file.
        /// </summary>
        public bool ContinueOnError
        {
            get { return false; }
        }

        /// <summary>
        /// Gets the column number of the task node within the project
        /// file that called it.
        /// </summary>
        public int ColumnNumberOfTaskNode
        {
            get { return 0; }
        }

        /// <summary>
        /// Gets the line number of the task node within the project file
        /// that called it.
        /// </summary>
        public int LineNumberOfTaskNode
        {
            get { return 0; }
        }

        /// <summary>
        /// Gets the full path to the project file that contained the call to this task.
        /// </summary>
        public string ProjectFileOfTaskNode
        {
            get { return string.Empty; }
        }

        /// <summary>
        /// Gets a value indicating whether the system is running in
        /// single- or multi-processor mode.
        /// </summary>
        public bool IsRunningMultipleNodes
        {
            get { return false; }
        }

        /// <summary>
        /// Initiates a build of a project file. If the build is successful, the outputs,
        /// if any, of the specified targets are returned.
        /// </summary>
        /// <param name="projectFileName">
        /// The name of the project file to build.
        /// </param>
        /// <param name="targetNames">
        /// The names of the target in the project to build.
        /// Can be <c>null</c>.
        /// </param>
        /// <param name="globalProperties">
        /// An IDictionary of additional global properties to apply to the project.
        /// The key and value must be <see cref="string"/> data types.
        /// </param>
        /// <param name="targetOutputs">
        /// The outputs of each specified target.
        /// Can be <c>null</c>.
        /// </param>
        /// <returns>
        /// <c>true</c> if the build was successful; otherwise, <c>false</c>.
        /// </returns>
        public bool BuildProjectFile(string projectFileName, string[] targetNames, IDictionary globalProperties, IDictionary targetOutputs)
        {
            return false;
        }

        /// <summary>
        /// Initiates a build of a project file. If the build is successful, the outputs,
        /// if any, of the specified targets are returned.
        /// </summary>
        /// <param name="projectFileName">
        /// The name of the project file to build.
        /// </param>
        /// <param name="targetNames">
        /// The names of the target in the project to build.
        /// Can be <c>null</c>.
        /// </param>
        /// <param name="globalProperties">
        /// An IDictionary of additional global properties to apply to the project.
        /// The key and value must be <see cref="string"/> data types.
        /// </param>
        /// <param name="targetOutputs">
        /// The outputs of each specified target.
        /// Can be <c>null</c>.
        /// </param>
        /// <param name="toolsVersion">
        /// Specifies the Toolset version for the build to use. 
        /// Can be <c>null</c> - The default value is "3.5.".
        /// </param>
        /// <returns>
        /// <c>true</c> if the build was successful; otherwise, <c>false</c>.
        /// </returns>
        public bool BuildProjectFile(string projectFileName, string[] targetNames, IDictionary globalProperties, IDictionary targetOutputs, string toolsVersion)
        {
            return false;
        }

        /// <summary>
        /// Initiates a build of a project file. If the build is successful, the outputs,
        /// if any, of the specified targets are returned.
        /// </summary>
        /// <param name="projectFileNames">
        /// The names of the project files to build.
        /// </param>
        /// <param name="targetNames">
        /// The names of the target in the project to build.
        /// Can be <c>null</c>.
        /// </param>
        /// <param name="globalProperties">
        /// An IDictionary of additional global properties to apply to the project.
        /// The key and value must be <see cref="string"/> data types.
        /// </param>
        /// <param name="targetOutputsPerProject">
        /// The outputs of each specified target.
        /// Can be <c>null</c>.
        /// </param>
        /// <param name="toolsVersion">
        /// Specifies the Toolset version for the build to use. 
        /// Can be <c>null</c> - The default value is "3.5.".
        /// </param>
        /// <param name="useResultsCache">
        /// If <c>true</c>, the operation runs only if the cache does not already contain the result.
        /// After the operation the result is stored in the cache.
        /// </param>
        /// <param name="unloadProjectsOnCompletion">
        /// If <c>true</c> the project is unloaded once the operation completes.
        /// </param>
        /// <returns>
        /// <c>true</c> if the build was successful; otherwise, <c>false</c>.
        /// </returns>
        public bool BuildProjectFilesInParallel(string[] projectFileNames, string[] targetNames, IDictionary[] globalProperties, IDictionary[] targetOutputsPerProject, string[] toolsVersion, bool useResultsCache, bool unloadProjectsOnCompletion)
        {
            return false;
        }

        /// <summary>
        /// Raises a custom event to the associated trace source
        /// as an information.
        /// </summary>
        /// <param name="e">
        /// The event data.
        /// </param>
        public void LogCustomEvent(CustomBuildEventArgs e)
        {
            Traces.Mock.TraceInformation(Message.MockLogCustom, e.Message);
        }

        /// <summary>
        /// Raises a message event to the associated trace source
        /// as an information.
        /// </summary>
        /// <param name="e">
        /// The event data.
        /// </param>
        public void LogMessageEvent(BuildMessageEventArgs e)
        {
            Traces.Mock.TraceInformation(Message.MockLogMessage, e.Message);
        }

        /// <summary>
        /// Raises a warning event to the associated trace source
        /// as a warning.
        /// </summary>
        /// <param name="e">
        /// The event data.
        /// </param>
        public void LogWarningEvent(BuildWarningEventArgs e)
        {
            Traces.Mock.TraceEvent(TraceEventType.Warning, 0, Message.MockLogWarning, e.Message);
        }

        /// <summary>
        /// Raises an error event to the associated trace source
        /// as an error.
        /// </summary>
        /// <param name="e">
        /// The event data.
        /// </param>
        public void LogErrorEvent(BuildErrorEventArgs e)
        {
            Traces.Mock.TraceEvent(TraceEventType.Error, 0, Message.MockLogError, e.Message);
        }
    }
}
