﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Encapsulates an apache based repository that simulates a remote repository
    /// with user/pass access.
    /// </summary>
    /// <remarks>
    /// This class is based on the following assemptions:
    /// <list type="bullet">
    /// <item>The secured repository is based on 'apache.zip'</item>
    /// <item>The secured repository is deployed in 'c:\apache'</item>
    /// <item>The secured repository is available on 'http://localhost:8080'.</item>
    /// <item>Apache is available in '%ProgramFiles%\Apache Software Foundation\Apache2.2'</item>
    /// </list>
    /// <para>This repository has been set-up by using the following references:</para>
    /// <list type="bullet">
    /// <item>http://mercurial.selenic.com/wiki/HgWebDirStepByStep</item>
    /// <item>http://blog.schuager.com/2010/03/how-to-setup-mercurial-server-on.html</item>
    /// </list>
    /// </remarks>
    public class MockRemoteRepository : IDisposable
    {
        /// <summary>
        /// The path to remote repository configuration.
        /// </summary>
        public const string RepositoryPath = "c:\\apache";

        /// <summary>
        /// The url of the repository.
        /// </summary>
        public const string RepositoryUrl = "http://localhost:8080/test";

        /// <summary>
        /// The path to the apache binaries.
        /// </summary>
        public const string ApachePath = "c:\\Program Files\\Apache Software Foundation\\Apache2.2\\bin";

        /// <summary>
        /// The name of the httpd execution binary.
        /// </summary>
        public const string HttpdExe = "httpd.exe";

        /// <summary>
        /// The parameters used to start apache.
        /// </summary>
        public const string HttpdParameters = "-w -f \"C:\\Apache\\httpd.conf\"";

        /// <summary>
        /// The timeout of the<see cref="MockRemoteRepository.Execute"/> command.
        /// </summary>
        private static int timeout = 5000;

        /// <summary>
        /// A value indicating whether the instance is initialized.
        /// </summary>
        private bool initialized;

        /// <summary>
        /// The encapsulated process.
        /// </summary>
        private Process process;

        /// <summary>
        /// The lines received on the standard output stream.
        /// </summary>
        private ICollection<string> outputs;

        /// <summary>
        /// The lines received on the standard error stream.
        /// </summary>
        private ICollection<string> errors;

        /// <summary>
        /// Initializes a new instance of the MockRemoteRepository class.
        /// </summary>
        public MockRemoteRepository()
        {
            this.errors = new LinkedList<string>();
            this.outputs = new LinkedList<string>();
        }

        /// <summary>
        /// Resets the test repository.
        /// </summary>
        public static void ResetRepository()
        {
            string testRepository = Path.Combine(RepositoryPath, "repos\\test");
            string initialRepository = Path.Combine(RepositoryPath, "repos\\initial");

            if (Directory.Exists(testRepository))
            {
                Directory.Delete(testRepository, true);
            }

            Utilities.DirectoryCopy(initialRepository, testRepository);
        }

        /// <summary>
        /// Resets the test repository to an empty state.
        /// </summary>
        public static void ResetEmptyRepository()
        {
            string testRepository = Path.Combine(RepositoryPath, "repos\\test");
            if (Directory.Exists(testRepository))
            {
                Directory.Delete(testRepository, true);
            }

            Directory.CreateDirectory(testRepository);
            Execute("init");
        }

        /// <summary>
        /// Executes an Hg command on the remote test repository.
        /// </summary>
        /// <param name="command">
        /// The executed command.
        /// </param>
        /// <returns>
        /// The lines of the standard output.
        /// </returns>
        /// <exception cref="AssertionException">
        /// The command failed.
        /// </exception>
        /// <seealso cref="HgProcess" />
        public static string[] Execute(string command)
        {
            string testRepository = Path.Combine(RepositoryPath, "repos\\test");
            using (HgProcess process = new HgProcess(null, command, testRepository))
            {
                process.StartAndWait(timeout);
                string errors = string.Join("\n", process.Errors.ToArray());
                Assert.AreEqual(0, process.Errors.Count, Message.RepositoryError, errors);

                return process.Outputs.ToArray();
            }
        }

        /// <summary>
        /// Initializes the http repository.
        /// </summary>
        /// <exception cref="NotSupportedException">
        /// The computer is not properly configured to start the remote repository.
        /// </exception>
        public void Initialize()
        {
            lock (this)
            {
                if (!Directory.Exists(RepositoryPath))
                {
                    string format = "The repository configuration cannot be found <{0}>";
                    string message = string.Format(CultureInfo.InvariantCulture, format, RepositoryPath);
                    throw new NotSupportedException(message);
                }

                if (!Directory.Exists(ApachePath))
                {
                    string format = "The apache 2.2 directory cannot be found <{0}>";
                    string message = string.Format(CultureInfo.InvariantCulture, format, ApachePath);
                    throw new NotSupportedException(message);
                }

                string processFileName = Path.Combine(ApachePath, HttpdExe);
                if (!File.Exists(processFileName))
                {
                    string format = "The apache 2.2 executable cannot be found <{0}>";
                    string message = string.Format(CultureInfo.InvariantCulture, format, processFileName);
                    throw new NotSupportedException(message);
                }

                if (this.initialized)
                {
                    string message = "The repository is already initialized";
                    throw new NotSupportedException(message);
                }

                ResetRepository();

                // Initialize the info about the process.
                ProcessStartInfo startInfo = new ProcessStartInfo(processFileName, HttpdParameters);
                startInfo.CreateNoWindow = true;
                startInfo.RedirectStandardError = true;
                startInfo.RedirectStandardOutput = true;
                startInfo.UseShellExecute = false;
                startInfo.WorkingDirectory = RepositoryPath;

                // Initialize the process
                this.process = new Process();
                this.process.StartInfo = startInfo;
                this.process.OutputDataReceived += this.OnOutputDataReceived;
                this.process.ErrorDataReceived += this.OnErrorDataReceived;

                try
                {
                    this.process.Start();
                    this.initialized = true;
                }
                catch (Win32Exception exception)
                {
                    this.process = null;
                    string message = "The http repository hasn't start properly";
                    throw new HgProcessException(message, exception);
                }

                this.process.BeginOutputReadLine();
                this.process.BeginErrorReadLine();
            }
        }

        /// <summary>
        /// Releases the encapsulated process.
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Called when the associated process send a message
        /// to its standard output stream.
        /// </summary>
        /// <param name="sender">
        /// This parameter is not used.
        /// </param>
        /// <param name="e">
        /// The message send to the standard stream.
        /// </param>
        private void OnOutputDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data != null)
            {
                lock (this.outputs)
                {
                    this.outputs.Add(e.Data);
                }
            }
        }

        /// <summary>
        /// Called when the associated process send a message
        /// to its standard error stream.
        /// </summary>
        /// <param name="sender">
        /// This parameter is not used.
        /// </param>
        /// <param name="e">
        /// The message send to the standard stream.
        /// </param>
        private void OnErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data != null)
            {
                lock (this.errors)
                {
                    this.errors.Add(e.Data);
                }
            }
        }

        /// <summary>
        /// Releases the encapsulated process.
        /// </summary>
        /// <param name="disposing">
        /// A value indicating whether the managed resource should
        /// be released.
        /// </param>
        /// <remarks>
        /// The encapsulated process is considered as a
        /// managed resource.
        /// </remarks>
        private void Dispose(bool disposing)
        {
            if (disposing == true)
            {
                if (this.process != null)
                {
                    this.process.Dispose();
                }
            }
        }
    }
}
