﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// A set of Assert methods operating on HgProcess outputs.
    /// </summary>
    /// <seealso cref="Repository.Execute"/>
    internal static class OutputAssert
    {
        /// <summary>
        /// Tests that at least one line of an output that starts
        /// with a specific prefix contains an expected value.
        /// </summary>
        /// <param name="output">
        /// The output of the <see cref="Repository.Execute"/> method.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the checked lines.
        /// </param>
        /// <param name="expected">
        /// The expected value the checked lines.
        /// </param>
        /// <remarks>
        /// If the <paramref name="output"/> contains no line that
        /// starts with <paramref name="prefix"/>, an Assertion will
        /// be raised.
        /// </remarks>
        public static void Contains(string[] output, string prefix, string expected)
        {
            bool tested = false;
            foreach (string line in output)
            {
                if (line.StartsWith(prefix, StringComparison.Ordinal))
                {
                    if (line.Contains(expected))
                    {
                        return;
                    }

                    tested = true;
                }
            }

            if (tested)
            {
                Assert.Fail(Message.OutputAssertNoContains2, prefix, expected);
            }
            else
            {
                Assert.Fail(Message.OutputAssertNoLines, prefix);
            }
        }

        /// <summary>
        /// Tests that at least one line of an output contains an expected value.
        /// </summary>
        /// <param name="output">
        /// The output of the <see cref="Repository.Execute"/> method.
        /// </param>
        /// <param name="expected">
        /// The expected value.
        /// </param>
        public static void Contains(string[] output, string expected)
        {
            foreach (string line in output)
            {
                if (line.Contains(expected))
                {
                    return;
                }
            }

            Assert.Fail(Message.OutputAssertNoContains1, expected);
        }

        /// <summary>
        /// Tests that all the lines starting with a specific prefix
        /// doesn't contain an expected value.
        /// </summary>
        /// <param name="output">
        /// The output of the <see cref="Repository.Execute"/> method.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the checked line.
        /// </param>
        /// <param name="expected">
        /// The checked value.
        /// </param>
        public static void DoesNotContain(string[] output, string prefix, string expected)
        {
            foreach (string line in output)
            {
                if (line.StartsWith(prefix, StringComparison.Ordinal))
                {
                    StringAssert.DoesNotContain(expected, line);
                }
            }
        }
    }
}
