﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Helper methods to manage a repository for testing purposes.
    /// </summary>
    internal static class Repository
    {
        /// <summary>
        /// The name of the unique file of the created repository.
        /// </summary>
        public const string FileName = "file.txt";

        /// <summary>
        /// The base path for the main repository used for the tests.
        /// </summary>
        private const string RepositoryPath = "temp\\repository";

        /// <summary>
        /// The base path for the second repository used for the tests.
        /// </summary>
        private const string RepositoryPath2 = "temp\\repository2";

        /// <summary>
        /// The timeout for the operations on the test repository
        /// - in milliseconds.
        /// </summary>
        private const int Timeout = 5 * 60 * 1000;

        /// <summary>
        /// Gets the full path of the main repository.
        /// </summary>
        public static string FullPath
        {
            get { return Path.GetFullPath(RepositoryPath); }
        }

        /// <summary>
        /// Gets the full path of the second repository.
        /// </summary>
        public static string FullPath2
        {
            get { return Path.GetFullPath(RepositoryPath2); }
        }

        /// <summary>
        /// Gets the full path of the unique file of the repository.
        /// </summary>
        public static string FilePath
        {
            get { return Path.Combine(RepositoryPath, FileName); }
        }

        /// <summary>
        /// Creates the main test repository with an initial commit.
        /// </summary>
        /// <remarks>
        /// This repository is configured with a default user
        /// and a first changeset.
        /// </remarks>
        public static void Create()
        {
            string path = FullPath;
            if (Directory.Exists(path))
            {
                Directory.Delete(path, true);
            }

            Directory.CreateDirectory(path);
            Repository.Execute("init");

            string hgrcFile = Path.Combine(path, ".hg/hgrc");
            File.AppendAllText(hgrcFile, "[ui]\r\nusername = default test user");

            string filePath = Path.Combine(path, FileName);
            File.WriteAllText(filePath, "test");
            Execute("add " + FileName);
            Execute("commit -m \"test commit 1\"");
        }

        /// <summary>
        /// Creates the second repository with no data.
        /// </summary>
        public static void Create2()
        {
            string path = FullPath2;
            if (Directory.Exists(path))
            {
                Directory.Delete(path, true);
            }

            Directory.CreateDirectory(path);
            Repository.Execute2("init");
        }

        /// <summary>
        /// Deletes the test repositories.
        /// </summary>
        public static void Delete()
        {
            Repository.Delete(Repository.FullPath);
            Repository.Delete(Repository.FullPath2);
        }

        /// <summary>
        /// Executes an Hg command on the main test repository.
        /// </summary>
        /// <param name="command">
        /// The executed command.
        /// </param>
        /// <returns>
        /// The lines of the standard output.
        /// </returns>
        /// <exception cref="AssertionException">
        /// The command failed.
        /// </exception>
        /// <seealso cref="HgProcess" />
        public static string[] Execute(string command)
        {
            using (HgProcess process = new HgProcess(null, command, FullPath))
            {
                process.StartAndWait(Timeout);
                string errors = string.Join("\n", process.Errors.ToArray());
                Assert.AreEqual(0, process.Errors.Count, Message.RepositoryError, errors);

                return process.Outputs.ToArray();
            }
        }

        /// <summary>
        /// Executes an Hg command on the second test repository.
        /// </summary>
        /// <param name="command">
        /// The executed command.
        /// </param>
        /// <returns>
        /// The lines of the standard output.
        /// </returns>
        /// <exception cref="AssertionException">
        /// The command failed.
        /// </exception>
        /// <seealso cref="HgProcess" />
        public static string[] Execute2(string command)
        {
            using (HgProcess process = new HgProcess(null, command, FullPath2))
            {
                process.StartAndWait(Timeout);
                string errors = string.Join("\n", process.Errors.ToArray());
                Assert.AreEqual(0, process.Errors.Count, Message.RepositoryError, errors);

                return process.Outputs.ToArray();
            }
        }

        /// <summary>
        /// Deletes a repository.
        /// </summary>
        /// <param name="path">
        /// The path of the repository.
        /// </param>
        private static void Delete(string path)
        {
            if (Directory.Exists(path))
            {
                // Delete all the files because Directory.Delete(path, true)
                // only works if there is no read only files.
                foreach (string file in Directory.GetFiles(path, "*", SearchOption.AllDirectories))
                {
                    try
                    {
                        File.Delete(file);
                    }
                    catch (IOException exception)
                    {
                        Traces.Default.TraceEvent(TraceEventType.Warning, 0, exception.Message);
                    }
                    catch (UnauthorizedAccessException exception)
                    {
                        Traces.Default.TraceEvent(TraceEventType.Warning, 0, exception.Message);
                    }
                }

                // Delete all the directories of the repository.
                Directory.Delete(path, true);
            }
        }
    }
}
