﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Initializes the remote repository.
    /// </summary>
    [SetUpFixture]
    public sealed class Setup : IDisposable
    {
        /// <summary>
        /// The unique instance of the <see cref="MockRemoteRepository"/> class.
        /// </summary>
        private MockRemoteRepository instance;

        /// <summary>
        /// Initializes the remote repository for all the tests.
        /// </summary>
        /// <exception cref="NotSupportedException">
        /// The remote repository is already initialized.
        /// </exception>
        /// <remarks>
        /// <para>
        /// This method is called by the NUnit framework and should not be used directly.
        /// </para>
        /// </remarks>
        [SetUp]
        public void SetUp()
        {
            if (this.instance == null)
            {
                try
                {
                    this.instance = new MockRemoteRepository();
                    this.instance.Initialize();
                }
                catch (Exception exception)
                {
                    Console.WriteLine(exception.Message);
                    throw;
                }
            }
            else
            {
                string message = "The remote repository is already initialized";
                throw new NotSupportedException(message);
            }
        }

        /// <summary>
        /// Closes the remote repository.
        /// </summary>
        /// <remarks>
        /// <para>
        /// This method is called by the NUnit framework and should not be used directly.
        /// </para>
        /// </remarks>
        [TearDown]
        public void TearDown()
        {
            if (this.instance != null)
            {
                this.instance.Dispose();
                this.instance = null;
            }
        }

        /// <summary>
        /// Disposes the encapsulated resources.
        /// </summary>
        public void Dispose()
        {
            this.TearDown();
        }
    }
}
