﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Globalization;
using System.IO;

namespace MSBuild.Mercurial
{
    /// <summary>
    /// Utilities methods for taking purposes.
    /// </summary>
    public static class Utilities
    {
        /// <summary>
        /// Copy a directory to another location.
        /// </summary>
        /// <param name="sourcePath">
        /// The path of the copied directory.
        /// </param>
        /// <param name="destinationPath">
        /// The path where the directory should be copied.
        /// </param>
        public static void DirectoryCopy(string sourcePath, string destinationPath)
        {
            DirectoryInfo source = new DirectoryInfo(sourcePath);
            DirectoryInfo destination = new DirectoryInfo(destinationPath);

            DirectoryCopy(source, destination);
        }

        /// <summary>
        /// Copy a directory to another location.
        /// </summary>
        /// <param name="source">
        /// The copied directory.
        /// </param>
        /// <param name="destination">
        /// The directory where the <paramref name="source"/> directory should be copied.
        /// </param>
        public static void DirectoryCopy(DirectoryInfo source, DirectoryInfo destination)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }

            if (destination == null)
            {
                throw new ArgumentNullException("destination");
            }

            if (!source.Exists)
            {
                string format = "The directory <{0}> doesn't exist";
                string message = string.Format(CultureInfo.InvariantCulture, format, source.FullName);
                throw new DirectoryNotFoundException(message);
            }

            if (destination.Exists)
            {
                string format = "The directory <{0}> already exist";
                string message = string.Format(CultureInfo.InvariantCulture, format, destination.FullName);
                throw new IOException(message);
            }

            // Create the destination directory.
            destination.Create();

            // Create the hierarchy of directories.
            foreach (DirectoryInfo directory in source.GetDirectories())
            {
                string subDirectoryPath = Path.Combine(destination.FullName, directory.Name);
                DirectoryInfo subDirectory = new DirectoryInfo(subDirectoryPath);
                DirectoryCopy(directory, subDirectory);
            }

            // Copy the files.
            foreach (FileInfo file in source.GetFiles())
            {
                string destFileName = Path.Combine(destination.FullName, file.Name);
                file.CopyTo(destFileName);
            }
        }
    }
}
