﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace MSBuild.Mercurial.Sample
{
    /// <summary>
    /// Manages and encapsulates the call of an external process.
    /// </summary>
    internal class ExternProcess : IDisposable
    {
        /// <summary>
        /// The working directory for the created process.
        /// </summary>
        private readonly string workingDirectory;

        /// <summary>
        /// The executed command.
        /// </summary>
        private readonly string command;

        /// <summary>
        /// The parameters of the command.
        /// </summary>
        private readonly string parameters;

        /// <summary>
        /// The encapsulated process.
        /// </summary>
        private Process process;

        /// <summary>
        /// The lines received on the standard output stream.
        /// </summary>
        private ICollection<string> outputs;

        /// <summary>
        /// The lines received on the standard error stream.
        /// </summary>
        private ICollection<string> errors;

        /// <summary>
        /// Initializes a new instance of the ExternProcess class.
        /// </summary>
        /// <param name="workingDirectory">
        /// The working directory for the created process.
        /// </param>
        /// <param name="command">
        /// The executed command.
        /// </param>
        /// <param name="parameters">
        /// The parameters of the command.
        /// </param>
        /// <exception cref="ArgumentNullException">
        /// One of the parameters <paramref name="command"/> or
        /// <paramref name="workingDirectory"/> is <c>null</c>.
        /// </exception>
        public ExternProcess(string workingDirectory, string command, string parameters)
        {
            if (workingDirectory == null)
            {
                throw new ArgumentNullException("workingDirectory");
            }

            if (command == null)
            {
                throw new ArgumentNullException("command");
            }

            this.command = command;
            this.workingDirectory = workingDirectory;
            this.parameters = parameters;

            this.outputs = new LinkedList<string>();
            this.errors = new LinkedList<string>();

            this.Start();
        }

        /// <summary>
        /// Gets the working directory for the created process.
        /// </summary>
        public string WorkingDirectory
        {
            get { return this.workingDirectory; }
        }

        /// <summary>
        /// Gets the executed command.
        /// </summary>
        public string Command
        {
            get { return this.command; }
        }

        /// <summary>
        /// Gets the parameters of the command.
        /// </summary>
        public string Parameters
        {
            get { return this.parameters; }
        }

        /// <summary>
        /// Gets the lines received on the standard output stream
        /// of the process.
        /// </summary>
        public ICollection<string> Outputs
        {
            get { return this.outputs; }
        }

        /// <summary>
        /// Gets the lines received on the standard error stream
        /// of the process.
        /// </summary>
        public ICollection<string> Errors
        {
            get { return this.errors; }
        }

        /// <summary>
        /// Waits for the end of the process.
        /// </summary>
        /// <returns>
        /// The exit code of the process.
        /// </returns>
        public int WaitForExit()
        {
            this.process.WaitForExit();
            return this.process.ExitCode;
        }

        /// <summary>
        /// Releases the encapsulated process.
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Starts the process.
        /// </summary>
        private void Start()
        {
            // Initialize the info about the process.
            ProcessStartInfo startInfo = new ProcessStartInfo(this.Command, this.Parameters);
            startInfo.CreateNoWindow = true;
            startInfo.RedirectStandardError = true;
            startInfo.RedirectStandardOutput = true;
            startInfo.UseShellExecute = false;
            startInfo.WorkingDirectory = this.WorkingDirectory;

            // Initialize the process
            this.process = new Process();
            this.process.StartInfo = startInfo;
            this.process.OutputDataReceived += new DataReceivedEventHandler(this.OnOutputDataReceived);
            this.process.ErrorDataReceived += new DataReceivedEventHandler(this.OnErrorDataReceived);

            this.process.Start();

            this.process.BeginOutputReadLine();
            this.process.BeginErrorReadLine();
        }

        /// <summary>
        /// Releases the encapsulated process.
        /// </summary>
        /// <param name="disposing">
        /// A value indicating whether the managed resource should
        /// be released.
        /// </param>
        /// <remarks>
        /// The encapsulated process is considered as a
        /// managed resource.
        /// </remarks>
        private void Dispose(bool disposing)
        {
            if (disposing == true)
            {
                if (this.process != null)
                {
                    this.process.Dispose();
                }
            }
        }

        /// <summary>
        /// Called when the associated process send a message
        /// to its standard output stream.
        /// </summary>
        /// <param name="sender">
        /// This parameter is not used.
        /// </param>
        /// <param name="e">
        /// The message send to the standard stream.
        /// </param>
        private void OnOutputDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data != null)
            {
                lock (this.outputs)
                {
                    this.outputs.Add(e.Data);
                }
            }
        }

        /// <summary>
        /// Called when the associated process send a message
        /// to its standard error stream.
        /// </summary>
        /// <param name="sender">
        /// This parameter is not used.
        /// </param>
        /// <param name="e">
        /// The message send to the standard stream.
        /// </param>
        private void OnErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data != null)
            {
                lock (this.errors)
                {
                    this.errors.Add(e.Data);
                }
            }
        }
    }
}
