﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using NUnit.Framework;

namespace MSBuild.Mercurial.Sample
{
    /// <summary>
    /// Tests the <c>HgPublish.targets</c> sample script.
    /// </summary>
    /// <remarks>
    /// This test creates two linked repositories and
    /// executes the HgPublish script.
    /// </remarks>
    [TestFixture]
    public static class HgPublishTest
    {
        /// <summary>
        /// The name of the project file.
        /// </summary>
        public const string ProjectFileName = "HgPublish.targets";

        /// <summary>
        /// The full path of the initial repository.
        /// </summary>
        public const string InitialRepository
            = Reference.SampleDirectory + "\\Initial";

        /// <summary>
        /// The full path of the cloned repository.
        /// </summary>
        public const string WorkingCopyRepository
            = Reference.SampleDirectory + "\\WorkingCopy";

        /// <summary>
        /// The full path of the result file.
        /// </summary>
        public const string ResultFilePath
            = WorkingCopyRepository + "\\Test.txt";

        /// <summary>
        /// Initializes the initial repository and its clone.
        /// </summary>
        [SetUp]
        public static void SetUp()
        {
            HgPublishTest.TearDown();

            Directory.CreateDirectory(InitialRepository);
            Directory.CreateDirectory(WorkingCopyRepository);

            Execute(InitialRepository, "hg", "init");
            File.WriteAllText(Path.Combine(InitialRepository, "Test.txt"), "text");
            Execute(InitialRepository, "hg", "add Test.txt");
            Execute(InitialRepository, "hg", "commit -m \"test commit\" -u test");

            Execute(WorkingCopyRepository, "hg", "clone ..\\Initial .");
        }

        /// <summary>
        /// Deletes the repositories.
        /// </summary>
        [TearDown]
        public static void TearDown()
        {
            if (Directory.Exists(InitialRepository))
            {
                Directory.Delete(InitialRepository, true);
            }

            if (Directory.Exists(WorkingCopyRepository))
            {
                Directory.Delete(WorkingCopyRepository, true);
            }
        }

        /// <summary>
        /// Runs the script and validates the generated file.
        /// </summary>
        /// <remarks>
        /// This test adds the script to the ignore list to create a
        /// new commit and runs the script.
        /// </remarks>
        [Test]
        [RequiresSTA]
        public static void Run()
        {
            Traces.Default.TraceInformation("HgPublishTest.Run");

            // Copy the script to the cloned repository
            string originalFile = Path.Combine(Reference.SampleDirectory, ProjectFileName);
            string projectFile = Path.Combine(WorkingCopyRepository, ProjectFileName);
            File.Copy(originalFile, projectFile);

            // Add the script in the ignore list.
            File.WriteAllText(Path.Combine(WorkingCopyRepository, ".hgignore"), "glob:*/" + ProjectFileName);
            Execute(WorkingCopyRepository, "hg", "add .hgignore");
            Execute(WorkingCopyRepository, "hg", "commit -m \"test update\"");

            // Execute the script
            int exitCode = MSBuildHelper.Execute(WorkingCopyRepository, ProjectFileName);
            Assert.AreEqual(0, exitCode);

            // Update the initial repository
            Execute(InitialRepository, "hg", "update");

            // Valid the tag
            ICollection<string> initialSummary = Execute(InitialRepository, "hg", "log -r1");
            bool tested = false;
            foreach (string line in initialSummary)
            {
                if (line.StartsWith("tag:", StringComparison.Ordinal))
                {
                    StringAssert.EndsWith("0.1.0", line);
                    tested = true;
                }
            }

            Assert.IsTrue(tested);
        }

        /// <summary>
        /// Executes a command on a specific Mercurial repository.
        /// </summary>
        /// <param name="repository">
        /// The path of the repository.
        /// </param>
        /// <param name="command">
        /// The command to execute.
        /// </param>
        /// <param name="parameters">
        /// The parameters of the command.
        /// </param>
        /// <returns>
        /// The output lines of the command.
        /// </returns>
        /// <remarks>
        /// This method includes an Assert test on the command exit
        /// code that checks its success.
        /// </remarks>
        private static ICollection<string> Execute(string repository, string command, string parameters)
        {
            ExternProcess process = new ExternProcess(repository, command, parameters);
            int exitCode = process.WaitForExit();

            if (exitCode != 0)
            {
                foreach (string line in process.Errors)
                {
                    Traces.Default.TraceEvent(TraceEventType.Warning, 0, line);
                }

                Assert.Fail();
            }

            return process.Outputs;
        }
    }
}
