﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.IO;
using System.Text.RegularExpressions;
using NUnit.Framework;

namespace MSBuild.Mercurial.Sample
{
    /// <summary>
    /// Tests the <c>HgVersion.targets</c> sample script.
    /// </summary>
    [TestFixture]
    public static class HgVersionTest
    {
        /// <summary>
        /// The path of the project file.
        /// </summary>
        public const string ProjectFileName = "HgVersion.targets";

        /// <summary>
        /// The path of the result file.
        /// </summary>
        public const string ResultFilePath
            = Reference.SampleDirectory + "\\AssemblyInfo.Generated.cs";

        /// <summary>
        /// The regular expression for the AssemblyVersion attribute
        /// generated in the result file.
        /// </summary>
        public const string AssemblyRegex = "\\[assembly: AssemblyVersion\\(\"([0-9\\.]+)\"\\)\\]";

        /// <summary>
        /// The regular expression for the AssemblyFileVersion attribute
        /// generated in the result file.
        /// </summary>
        public const string AssemblyFileRegex = "\\[assembly: AssemblyFileVersion\\(\"([0-9\\.]+)\"\\)\\]";

        /// <summary>
        /// Runs the script and validates the generated file.
        /// </summary>
        [Test]
        [RequiresSTA]
        public static void Run()
        {
            Traces.Default.TraceInformation("HgVersionTest.Run");
            
            int exitCode = MSBuildHelper.Execute(ProjectFileName);

            Assert.AreEqual(0, exitCode);
            Assert.IsTrue(File.Exists(ResultFilePath));

            using (TextReader reader = File.OpenText(ResultFilePath))
            {
                string line = reader.ReadLine();

                string assemblyVersion = null;
                string assemblyFileVersion = null;

                while (line != null)
                {
                    Match assemblyMatch = Regex.Match(line, AssemblyRegex);
                    Match assemblyFileMatch = Regex.Match(line, AssemblyFileRegex);

                    if (assemblyMatch.Success)
                    {
                        assemblyVersion = assemblyMatch.Groups[1].Value;
                    }

                    if (assemblyFileMatch.Success)
                    {
                        assemblyFileVersion = assemblyFileMatch.Groups[1].Value;
                    }

                    line = reader.ReadLine();
                }

                StringAssert.StartsWith("0.1.0.", assemblyVersion);
                StringAssert.StartsWith("0.1.0.", assemblyFileVersion);

                int expectedRevision = typeof(HgVersionTest).Assembly.GetName().Version.Revision;
                Assert.AreNotEqual(0, expectedRevision);
                Version actual = new Version(assemblyVersion);
                Assert.AreEqual(expectedRevision, actual.Revision);
            }
        }
    }
}
