﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Diagnostics;
using System.IO;
using System.Text;
using NUnit.Framework;

namespace MSBuild.Mercurial.Sample
{
    /// <summary>
    /// Helper methods that encapsulate MSBuild process.
    /// </summary>
    internal static class MSBuildHelper
    {
        /// <summary>
        /// The path to the MSBuild executable.
        /// </summary>
        public const string MSBuildPath
            = Reference.Framework35 + "\\msbuild.exe";

        /// <summary>
        /// Executes MSBuild for a specific sample project.
        /// </summary>
        /// <param name="projectFileName">
        /// The name of the project file, should be in the sample directory.
        /// </param>
        /// <returns>
        /// The exit code of the MSBuild process.
        /// </returns>
        public static int Execute(string projectFileName)
        {
            return Execute(Reference.SampleDirectory, projectFileName);
        }

        /// <summary>
        /// Executes MSBuild for a specific sample project.
        /// </summary>
        /// <param name="repository">
        /// The path of the repository.
        /// </param>
        /// <param name="projectFileName">
        /// The name of the project file, should be in the sample directory.
        /// </param>
        /// <returns>
        /// The exit code of the MSBuild process.
        /// </returns>
        public static int Execute(string repository, string projectFileName)
        {
            Assert.IsTrue(File.Exists(MSBuildPath));
            Assert.IsTrue(File.Exists(Path.Combine(repository, projectFileName)));

            // Define the start up info of the process
            string processFileName = MSBuildPath;
            ProcessStartInfo startInfo = new ProcessStartInfo(processFileName, projectFileName);
            startInfo.CreateNoWindow = true;
            startInfo.RedirectStandardError = true;
            startInfo.RedirectStandardOutput = true;
            startInfo.UseShellExecute = false;
            startInfo.WorkingDirectory = repository;
            startInfo.StandardErrorEncoding = Encoding.UTF8;
            startInfo.StandardOutputEncoding = Encoding.UTF8;

            // Initialize the process
            Process process = new Process();
            process.StartInfo = startInfo;
            process.ErrorDataReceived += new DataReceivedEventHandler(OnErrorDataReceived);
            process.OutputDataReceived += new DataReceivedEventHandler(OnOutputDataReceived);
            process.Start();
            process.BeginErrorReadLine();
            process.BeginOutputReadLine();
            process.WaitForExit();

            return process.ExitCode;
        }

        /// <summary>
        /// Called when a message is returned by the associated process
        /// to be logged.
        /// </summary>
        /// <param name="sender">
        /// The process that sends the message.
        /// </param>
        /// <param name="e">
        /// The message to be logged.
        /// </param>
        private static void OnOutputDataReceived(object sender, DataReceivedEventArgs e)
        {
            Traces.Default.TraceEvent(TraceEventType.Information, 0, e.Data);
        }

        /// <summary>
        /// Called when an error is returned by the associated process
        /// to be logged.
        /// </summary>
        /// <param name="sender">
        /// The process that sends the error.
        /// </param>
        /// <param name="e">
        /// The message to be logged.
        /// </param>
        private static void OnErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            Traces.Default.TraceEvent(TraceEventType.Warning, 0, e.Data);
        }
    }
}
